import { API_URL } from "$env/static/private";

import type { PageServerLoad } from "./$types";

import { redirect } from "@sveltejs/kit";

import type { Chapter, LeaderboardEvent } from "$lib/types";
import { handleRedirect } from "$lib/utils";

export const load: PageServerLoad = async ({ url, locals, fetch, params: { chapterId } }) => {
  if (!locals.user) {
    redirect(302, handleRedirect("login", url));
  }

  let chapter: Chapter | null = null;

  let res = await fetch(API_URL + "/chapter/" + chapterId);

  if (!res.ok) {
    redirect(302, "/chapters");
  }

  chapter = await res.json();

  if (!(chapter?.start && chapter?.end)) redirect(302, "/chapters/" + chapterId);

  let leaderboard: LeaderboardEvent | null = null;

  res = await fetch(API_URL + `/leaderboard/${chapterId}`);

  if (res.ok) {
    leaderboard = await res.json();
  }

  return {
    title: 'Classement' + ' - ' + chapter.name,
    
    chapter,
    leaderboard,
  };
};
