import { API_URL } from "$env/static/private";

import type { Actions, PageServerLoad } from "./$types";

import { redirect } from "@sveltejs/kit";

import { compile } from 'mdsvex';
import { superValidate } from "sveltekit-superforms";
import { zod } from "sveltekit-superforms/adapters";

import { COOKIES } from "$lib/constants";
import type { Chapter } from "$lib/types";
import { handleRedirect } from "$lib/utils";

import { formSchema } from "./schema";

export const load: PageServerLoad = async ({ url, locals, fetch, cookies, params: { chapterId, puzzleId } }) => {
  if (!locals.user) {
    redirect(302, handleRedirect("login", url));
  }

  const session = cookies.get(COOKIES.SESSION);

  let chapter: Chapter | null = null;

  let res = await fetch(API_URL + "/chapter/" + chapterId);

  if (!res.ok) {
    redirect(302, "/chapters");
  }

  chapter = await res.json();

  if (!chapter || !chapter.show && !(chapter.start && chapter.end)) {
    redirect(302, "/chapters");
  }

  if (!chapter.puzzles.length) {
    redirect(302, "/chapters/" + chapterId);
  }

  let puzzle = chapter.puzzles.find((p) => p.id === parseInt(puzzleId));

  if (!puzzle) {
    redirect(302, "/chapters/" + chapterId);
  }

  res = await fetch(API_URL + "/puzzle/" + puzzleId);

  if (!res.ok) {
    redirect(302, "/chapters/" + chapterId);
  }

  puzzle = await res.json();

  if (!puzzle) {
    redirect(302, "/chapters/" + chapterId);
  }

  const content = await compile(puzzle.content);

  if (content)
    puzzle.content = content?.code
      .replace(/>{@html `<code class="language-/g, '><code class="language-')
      .replace(/<\/code>`}<\/pre>/g, '</code></pre>');

  return {
    title: puzzle.name + " - " + chapter.name,

    chapter,
    puzzle,
    session,
    url: API_URL + "/puzzleResponse/" + puzzleId,
    form: await superValidate(zod(formSchema))
  };
};

export const actions: Actions = {
  default: async ({ request, fetch, params: { chapterId, puzzleId } }) => {

    const formData = await request.formData();

    const res = await fetch(`${API_URL}/puzzleResponse/${puzzleId}`, {
      method: 'POST',
      body: formData
    });

    if (!res.ok) {
      console.log(res)
      return;
    }

    redirect(302, `/chapters/${chapterId}/puzzle/${puzzleId}`);
  }
}

