import { dev } from "$app/environment";
import { API_URL } from "$env/static/private";

import type { Actions, PageServerLoad } from "./$types";

import { redirect } from "@sveltejs/kit";

import { fail, setError, superValidate } from "sveltekit-superforms";
import { zod } from 'sveltekit-superforms/adapters';

import { COOKIES } from "$lib/constants";

import { formSchema } from "./schema";

export const load: PageServerLoad = async ({ locals }) => {
  if (locals.user) {
    redirect(302, "/");
  }

  return {
    title: 'Connexion',

    form: await superValidate(zod(formSchema))
  }
};

export const actions: Actions = {
  default: async ({ request, locals, fetch, cookies, url }) => {
    if (locals.user) {
      redirect(302, "/");
    }

    const form = await superValidate(request, zod(formSchema));

    if (!form.valid) {
      return fail(400, {
        form,
      });
    }

    try {
      const res = await fetch(API_URL + "/login", {
        method: "POST",
        body: JSON.stringify({ ...form.data }),
      });

      if (!res.ok) {
        return setError(form, 'passwd', "Nom d'utilisateur ou mot de passe incorrect");
      }

      const token = res.headers.get("Authorization")?.split("Bearer ").pop();

      if (!token) {
        return setError(form, 'passwd', "Une erreur est survenue, veuillez réessayer");
      }

      cookies.set(COOKIES.SESSION, token, {
        path: "/",
        secure: !dev,
        sameSite: "strict",
      });
    } catch (error) {
      console.error(error);
      return setError(form, 'passwd', "Une erreur est survenue, veuillez réessayer");
    }

    const redirectTo = url.searchParams.get('redirectTo');

    if (redirectTo) {
      redirect(302, `/${redirectTo.slice(1)}`);
    }

    redirect(302, "/");
  },
};
